/*
 * Copyright 2018 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.kie.workbench.common.dmn.client.canvas.controls.keyboard.shortcut;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.runner.RunWith;
import org.kie.workbench.common.stunner.core.client.canvas.controls.keyboard.shortcut.AbstractAppendNodeShortcut;
import org.kie.workbench.common.stunner.core.client.canvas.controls.keyboard.shortcut.BaseAppendNodeShortcutTest;
import org.kie.workbench.common.stunner.core.client.event.keyboard.KeyboardEvent;
import org.kie.workbench.common.stunner.core.graph.Element;
import org.mockito.runners.MockitoJUnitRunner;

@RunWith(MockitoJUnitRunner.class)
public class AppendDecisionShortcutTest extends BaseAppendNodeShortcutTest {

    @Override
    public AbstractAppendNodeShortcut getTestedShortcut() {
        return new AppendDecisionShortcut(toolboxDomainLookups,
                                          definitionsCacheRegistry,
                                          generalCreateNodeAction);
    }

    @Override
    public List<KeyboardEvent.Key> getAcceptableKeys() {
        return Collections.singletonList(KeyboardEvent.Key.D);
    }

    @Override
    public Object getAcceptableTargetDefinition() {
        return DmnNode.DECISION.getDefinition();
    }

    @Override
    public List<Object> getNotAcceptableTargetDefinitions() {
        return Stream.of(DmnNode.values())
                .filter(node -> node.getDefinition() != getAcceptableTargetDefinition())
                .map(node -> node.getDefinition())
                .collect(Collectors.toList());
    }

    @Override
    public List<Element> getAcceptableSelectedElements() {
        return Arrays.asList(DmnNode.DECISION.getElement(),
                             DmnNode.INPUT_DATA.getElement());
    }

    @Override
    public List<Element> getNotAcceptableSelectedElements() {
        return Stream.of(DmnNode.values())
                .map(node -> node.getElement())
                .filter(element -> !getAcceptableSelectedElements().contains(element))
                .collect(Collectors.toList());
    }
}
