/*
 * Copyright 2020 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.kie.workbench.common.dmn.client.editors.expressions.util;

import org.junit.Test;

import static org.assertj.core.api.Assertions.assertThat;

public class NameUtilsTest {

    @Test
    public void testNull() {
        assertThat(NameUtils.normaliseName(null)).isEmpty();
    }

    @Test
    public void testEmpty() {
        assertThat(NameUtils.normaliseName("")).isEmpty();
    }

    @Test
    public void testLeadingWhitespace() {
        assertThat(NameUtils.normaliseName("  hello")).isEqualTo("hello");
    }

    @Test
    public void testTrailingWhitespace() {
        assertThat(NameUtils.normaliseName("hello   ")).isEqualTo("hello");
    }

    @Test
    public void testCompactWhitespace() {
        assertThat(NameUtils.normaliseName("hello   world")).isEqualTo("hello world");
    }

    @Test
    public void testCompleteNormalisation() {
        assertThat(NameUtils.normaliseName("  hello   world  ")).isEqualTo("hello world");
    }
}
